/***************************************************************************
 *            AlertsManager.cc
 *
 *  Copyright  2008  Fernando Tarín Morales
 *  icemanf@gmail.com
 ****************************************************************************/

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */
 
#include "Lince/AlertsManager.h"
#include "Lince/Core.h"
#include "Lince/Utils.h"

#include "Lince/Compose.h"
#include "../config.h"
#include "Lince/i18n.h"

using namespace Lince;
using namespace libtorrent;

AlertsManagerPtr AlertsManager::create()
{
	return AlertsManagerPtr(new AlertsManager());
}

AlertsManager::AlertsManager()
{
}

void AlertsManager::add_local_alert(const local_alert &la)
{
	local_alerts.push(la);
}

void AlertsManager::check_alerts()
{
	int type;
	std::auto_ptr<alert> a = Core::get_session_manager()->pop_alert();
	local_alert la;

	while (!local_alerts.empty())
	{
		std::string translated;
		la = local_alerts.front();
		if (la.type == TORRENT_ADDED || la.type == TORRENT_ADDED_PREVIOUS_SESSION)
		{
			if (la.type == TORRENT_ADDED)
				translated = String::compose(_("%1 added"), la.name);
			else
				translated = String::compose(_("%1 added from previous session"), la.name);

			torrentPosition position;
			TorrentsManagerPtr tm = Core::get_torrents_manager();

			position = tm->get_torrent_position(la.hash);

			m_signal_torrent_added(position);
			m_signal_torrent_added_log(translated, la.level);
		}
		if (la.type == TORRENT_REMOVED)
		{
			translated = String::compose(_("%1 removed"), la.name);
			m_signal_torrent_removed(translated, la.level);
		}
		if (la.type == QUEUED)
		{
			translated = String::compose(_("%1 queued"), la.name);
			m_signal_torrent_queued(translated, la.level);
		}
		if (la.type == UNQUEUED)
		{
			translated = String::compose(_("%1 unqueued"), la.name);
			m_signal_torrent_unqueued(translated, la.level);
		}
		if (la.type == STARTED)
		{
			translated = String::compose(_("%1 started"), la.name);
			torrentPosition position;
			TorrentsManagerPtr tm = Core::get_torrents_manager();

			position = tm->get_torrent_position(la.hash);

			m2_signal_torrent_started(position);
			m_signal_torrent_started(translated, la.level);
		}
		if (la.type == STOPPED)
		{
			translated = String::compose(_("%1 stopped"), la.name);
			torrentPosition position;
			TorrentsManagerPtr tm = Core::get_torrents_manager();

			position = tm->get_torrent_position(la.hash);

			m2_signal_torrent_stopped(position);
			m_signal_torrent_stopped(translated, la.level);
		}

		local_alerts.pop();
	}

	while (a.get())
	{
		try{
			type = NONE;
			
			if (listen_failed_alert* p = dynamic_cast<libtorrent::listen_failed_alert*>(a.get()))
			{
				std::string translated;
				translated = String::compose(_("Cannot bind to interface %1. Error: %2")
					,p->endpoint.address().to_string(), p->error.message());
				
				type = FATAL;
				m_signal_listen_failed(translated, type);
			}
			else if (torrent_finished_alert* p = dynamic_cast<torrent_finished_alert*>(a.get()))
			{
				torrentPosition position;
				TorrentsManagerPtr tm = Core::get_torrents_manager();

				position = tm->get_torrent_position(p->handle.info_hash());
				if (!position.second)
				{
					std::string translated = String::compose(_("%1 finished"), p->handle.name());
					type = INFO;
					m_signal_finished_torrent(position.first);
					m_signal_finished_torrent_log(translated, type);
				}
			}
			else if (portmap_error_alert* p = dynamic_cast<portmap_error_alert*>(a.get()))
			{
				if (p->type == 0)
					m_signal_portmap_failed(_("Failed to map NAT-PMP ports"), type);
				else
					m_signal_portmap_failed(_("Failed to map UPnP ports"), type);
				type = CRITICAL;
			}
			else if (torrent_paused_alert* p = dynamic_cast<torrent_paused_alert*>(a.get()))
			{
			}
			else if (portmap_alert* p = dynamic_cast<portmap_alert*>(a.get()))
			{
				// TODO: Revisar esto.
				std::string translated, msg = p->message();
				std::string port = msg.substr(29);
				if (str_has_prefix(msg, "successfully mapped UDP port "))
					translated = String::compose(_("Mapped UDP port %1"), port);
				else
					translated = String::compose(_("Mapped TCP port %1"), port);
				type = WARNING;
				m_signal_portmap_success(translated, type);
			}
			else if (file_error_alert* p = dynamic_cast<file_error_alert*>(a.get()))
			{
				std::string translated;
				TorrentsManagerPtr tm = Core::get_torrents_manager();
				torrentPosition position;
				position = tm->get_torrent_position(p->handle.info_hash());

				tm->get_torrent_at(position)->stop();

				translated = String::compose(_("File error: %1 %2"), p->file, p->msg);
				std::cout << "File error: " << p->file << " " << p->msg << std::endl;
				type = FATAL;
				m_signal_file_error(translated, type);
			}
			if (tracker_announce_alert* p = dynamic_cast<tracker_announce_alert*>(a.get()))
			{
				std::string translated;

				switch(p->event){
					case 0:
						translated = String::compose(_("%1 announcing"), p->handle.name());
						break;
					case 1:
						translated = String::compose(_("%1 announcing as finished"), p->handle.name());
						break;
					case 2:
						translated = String::compose(_("%1 announcing as started"), p->handle.name());
						break;
					case 3:
						translated = String::compose(_("%1 announcing as stopped"), p->handle.name());
						break;
				}

				type = INFO;
				m_signal_tracker_announce(translated, type);
			}
			else if (tracker_error_alert* p = dynamic_cast<tracker_error_alert*>(a.get()))
			{
				std::string translated, msg = p->message();
				std::string tracker = msg.substr(10, msg.find(" ", 9) - 11);

				switch (p->status_code)
				{
				case 0:
					translated = String::compose(_("%1 tracker timed out"), p->handle.name());
					break;
				case -1:
					translated = String::compose(_("%1 tracker connection refused"), p->handle.name());
					break;
				case 200:
					translated = msg;
					break;
				default:
					translated = String::compose(_("%1 tracker failed(%2)"), p->handle.name(), p->status_code);
					break;
				}

				type = WARNING;
				m_signal_tracker_failed(translated, type);
			}
			else if (tracker_reply_alert* p = dynamic_cast<tracker_reply_alert*>(a.get()))
			{
				std::string translated;

				translated = String::compose(_("%1 Tracker OK, got %2 peers"), p->handle.name(), p->num_peers);

				type=INFO;
				m_signal_tracker_reply(translated, type);
			}
			else if (tracker_warning_alert* p = dynamic_cast<tracker_warning_alert*>(a.get()))
			{
				// TODO: Revisar
				type=WARNING;
				m_signal_tracker_warning(p->message(), type);
			}
			else if (url_seed_alert* p = dynamic_cast<url_seed_alert*>(a.get()))
			{
				// TODO: Revisar
				type=WARNING;
				m_signal_http_seed_failed(p->message(), type);
			}
			else if (hash_failed_alert* p = dynamic_cast<hash_failed_alert*>(a.get()))
			{
				std::string translated = String::compose(_("%1 hash failed for piece %2"), p->handle.name(), p->piece_index);
				type=WARNING;
				m_signal_hash_failed(translated, type);
			}
			else if (peer_ban_alert* p = dynamic_cast<peer_ban_alert*>(a.get()))
			{
				std::string translated = String::compose(_("%1 banning peer %2"), p->handle.name(), p->ip.address().to_string());
				type=WARNING;
				m_signal_peer_banned(translated, type);
				m_signal_ban_peer(p->ip.address().to_string());
			}
			else if (fastresume_rejected_alert* p = dynamic_cast<fastresume_rejected_alert*>(a.get()))
			{
				std::string translated, tracker, msg = p->message();
				if (str_has_prefix(msg, "incompatible file version "))
				{
					std::string version = msg.substr(26);
					translated = String::compose(_("%1 incompatible file version: %2"), p->handle.name(), version);
				}
				else if (str_has_prefix(msg, "mismatching info-hash: "))
				{
					std::string hash = msg.substr(23);
					translated = String::compose(_("%1 mismatching hash: %2"), p->handle.name(), hash);
				}
				else if (str_has_prefix(msg, "checksum mismatch on piece "))
				{
					std::string index = msg.substr(27);
					translated = String::compose(_("%1 checksum failed for piece %2"), p->handle.name(), index);
				}
				else if (str_has_prefix(msg, "the number of files does not match the torrent"))
				{
					translated = String::compose(_("%1 mismatching number of files"), p->handle.name());
				}
				else if (str_has_prefix(msg, "file size for "))
				{
					std::string::size_type pos = msg.find(" was expected to be ");
					std::string file = msg.substr(14, pos);
					translated = String::compose(_("%1 file size mismatch for %2"), p->handle.name(), file);
				}
				else // Other too detailed errors
					translated = _("Fast resume rejected, content check forced");

				//TODO Make a signal to pass a finished torrent to downloading torrents
				type=CRITICAL;
				m_signal_fastresume_rejected(translated, type);
			}
			else if (storage_moved_alert* p = dynamic_cast<storage_moved_alert*>(a.get()))
			{
				m_signal_storage_moved(p->message(), type);
			}
			else if (torrent_checked_alert* p = dynamic_cast<torrent_checked_alert*>(a.get()))
			{
				std::string translated;
				translated = String::compose(_("%1 checked finished"), p->handle.name());
				type=INFO;
				m_signal_torrent_checked(translated, type);
			}
			else if (listen_succeeded_alert* p = dynamic_cast<listen_succeeded_alert*>(a.get()))
			{
				std::string translated;
				type=INFO;
				translated = String::compose(_("Listening on interface [%1]:%2"), p->endpoint.address(), p->endpoint.port());

				m_signal_listen_succeded(translated, type);
			}
			else if (save_resume_data_alert* rd = dynamic_cast<save_resume_data_alert*>(a.get()))
			{
				TorrentsManagerPtr tm = Core::get_torrents_manager();
				torrentPosition position;
				position = tm->get_torrent_position(rd->handle.info_hash());
				std::string name = tm->get_torrent_at(position)->name();

				boost::filesystem::path fname = build_filename(get_resume_data_base_directory(), name);

				boost::filesystem::ofstream out(fname, std::ios_base::binary);
				out.unsetf(std::ios_base::skipws);

				bencode(std::ostream_iterator<char>(out), *rd->resume_data);
			}
			// TODO: What to do with this?
			else
			{
			}
		}
		catch (libtorrent::invalid_handle)
		{
		}

		a = Core::get_session_manager()->pop_alert();
	}
}

AlertsManager::~AlertsManager()
{
}
