/***************************************************************************
 *            Torrent.h
 *
 *  Copyright  2008  Fernando Tarín Morales
 *  icemanf@gmail.com
 ****************************************************************************/

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */

#ifndef TORRENT_H
#define TORRENT_H

#include <string>
#include <vector>

#include <boost/filesystem.hpp>
#include <boost/filesystem/fstream.hpp>
#include <boost/date_time/posix_time/posix_time.hpp>
#include <boost/date_time/gregorian/gregorian.hpp>

#include "libtorrent/entry.hpp"
#include "libtorrent/torrent.hpp"
#include "libtorrent/alert_types.hpp"

#include "Timer.h"

namespace Lince
{

struct torrent_queued : public libtorrent::torrent_alert
{
};

struct torrent_localinfo
{
	int max_upload, max_download, max_connects;
	int max_upload_slots, storage_mode, failed_hashes;

	boost::int64_t total_uploaded, total_time;

	float share_ratio;

	bool is_complete, is_stopped;

	std::string file_name, save_path;

	std::vector<int> file_priorities;
	std::vector<std::string> file_new_names;
	std::vector<libtorrent::announce_entry> trackers;

	torrent_localinfo() : max_upload(-1), max_download(-1), max_connects(-1)\
		, max_upload_slots(-1), storage_mode(0), failed_hashes(0), total_uploaded(0)\
		, total_time(0), share_ratio(1.0), is_complete(false), is_stopped(false) {};
};

class Torrent
{
	private:
		int pos, failedHashes, uploadsSlots, maxConnects;
		int maxUpload, maxDownload;
		bool isQueued, isStopped, isComplete;
		boost::int64_t totalTime, totalUploaded;
		float shareRatio;
		Timer timer;
		std::string fileName;
		std::vector<int> filePriorities;
		std::vector<std::string> fileNewNames;
		libtorrent::torrent_handle th;
		libtorrent::storage_mode_t storageMode;

	public:
		Torrent();
		Torrent(const libtorrent::torrent_handle &);

		libtorrent::torrent_status status() const { return th.status(); };
		libtorrent::torrent_info const& get_torrent_info() const { return th.get_torrent_info(); };
		void get_peer_info(std::vector<libtorrent::peer_info> &pi) const { th.get_peer_info(pi); };

		void set_handle(const libtorrent::torrent_handle &_th) { th = _th; };
		libtorrent::torrent_handle & get_handle() { return th; };


		int position() const { return pos; };
		void set_position(int value) { pos = value; };

		bool is_complete() const { return isComplete; };
		void set_complete(bool value=true) { isComplete = value; };

		int failed_hashes()
		{
			return failedHashes +
				(status().total_failed_bytes / get_torrent_info().piece_length());
		};
		void set_failed_hashes(int value) { failedHashes = value; };

		libtorrent::storage_mode_t storage_mode() { return storageMode; };
		void set_storage_mode(libtorrent::storage_mode_t value) { storageMode = value; };

		int upload_slots() { return uploadsSlots; };
		void set_upload_slots(int value)
		{
			uploadsSlots = value;
			th.set_max_uploads(value);
		};

		int max_upload_speed() { return maxUpload; };
		void set_max_upload_speed(int speed)
		{
			maxUpload = speed;
			if (speed == -1)
				th.set_upload_limit(speed);
			else
				th.set_upload_limit(speed * 1024);
		};

		int max_download_speed() { return maxDownload; };
		void set_max_download_speed(int speed)
		{
			maxDownload = speed;
			if (speed == -1)
				th.set_download_limit(speed);
			else
				th.set_download_limit(speed * 1024);
		};

		int max_connects() { return maxConnects; };
		void set_max_connects(int value)
		{
			maxConnects = value;
			th.set_max_connections(value);
		};

		boost::int64_t elapsed_time() { return totalTime + timer.elapsed(); };
		void set_elapsed_time(boost::int64_t value) { totalTime = value; timer.reset(); };

		std::string file_name() const { return fileName; };
		void set_file_name(const std::string &fn) { fileName = fn; };

		bool is_stopped() { return isStopped; };
		void stop();
		void start();

		bool is_queued() { return isQueued; };
		void queue();
		void unqueue();

		boost::int64_t total_uploaded() { return (totalUploaded +
				static_cast<boost::int64_t>(status().total_payload_upload)); };
		void set_total_uploaded(boost::int64_t value) { totalUploaded = value; };

		float share_ratio() { return shareRatio; };
		void set_share_ratio(float value)
		{
			shareRatio = value;
			th.set_ratio(value);
		};

		std::vector<int> file_priorities() const { return filePriorities; };
		void set_file_priorities(const std::vector<int> &i) { filePriorities = i; };

		std::vector<std::string> file_new_names() const { return fileNewNames; };
		void set_file_new_names(const std::vector<std::string> &f) { fileNewNames= f; };

		void prioritize_first_part_of_files(int, boost::int64_t);
		libtorrent::entry create_torrent();
		void save_resume_data();

		// Query functions
		int get_peers(void) { return status().num_peers; };
		int get_seeders(void) { return status().num_seeds; };
		int get_downloaders(void) { return (status().num_peers - status().num_seeds); };
		int get_total_seeders(void) { return status().num_complete; };
		int get_total_downloaders(void) { return status().num_incomplete; };
		int get_total_pieces(void) { return get_torrent_info().num_pieces(); };
		int get_downloaded_pieces(void) { return status().num_pieces; };
		int get_priority_of_file(int);

		const libtorrent::bitfield get_pieces() { return status().pieces; };
		int num_pieces() { return status().num_pieces; };

		float get_distributed_copies(void) { return status().distributed_copies; };
		float get_download_speed(void) { return status().download_rate; };
		float get_upload_speed(void) { return status().upload_rate; };
		float get_percentage(void) { return (status().progress * 100); };
		float get_progress(void) { return status().progress; };
		float get_ratio(void);
		float get_eta(void);

		std::string get_status_string();
		std::string name() const { return th.name(); };

		libtorrent::size_type get_size(void) { return get_torrent_info().total_size(); };
		libtorrent::size_type get_downloaded(void) { return status().total_done; };
		libtorrent::size_type get_session_uploaded(void) { return total_uploaded(); };
		libtorrent::size_type get_remaining(void) { return (get_torrent_info().total_size() - status().total_done); };

		libtorrent::sha1_hash get_hash(void) { return get_torrent_info().info_hash(); };
		libtorrent::size_type get_piece_length(void) { return get_torrent_info().piece_length(); };
		std::string get_date_as_string(void);

		std::string get_comment(void) { return get_torrent_info().comment(); };

		libtorrent::torrent_status::state_t get_status(void) { return status().state; };

		libtorrent::sha1_hash info_hash() const { return th.info_hash(); };

		boost::posix_time::time_duration get_next_announce(void)
		{
			return status().next_announce;
		};

		boost::filesystem::path save_path() const { return th.save_path(); };

		std::vector<libtorrent::announce_entry> const& trackers() const { return th.trackers(); };
		void force_reannounce() const { th.force_reannounce(); };
		void replace_trackers(std::vector<libtorrent::announce_entry> const&t) const
		{
			th.replace_trackers(t);
		};
		void set_tracker_login(std::string const& username, std::string const& password) const
		{
			th.set_tracker_login(username, password);
		};

		void get_download_queue(std::vector<libtorrent::partial_piece_info>& queue) const
		{
			th.get_download_queue(queue);
		};

		void force_recheck() const { th.force_recheck(); };

		bool has_incoming() { return th.status().has_incoming; };
		bool tracker_connected() { return !th.status().current_tracker.empty(); };
};

} // Namespace

#endif // Torrent
