/***************************************************************************
 *            SetupManager.cc
 *
 *  Copyright  2008  Fernando Tarín Morales
 *  icemanf@gmail.com
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */

#include "Lince/SetupManager.h"
#include "Lince/Utils.h"

#include "../config.h"
#include "Lince/i18n.h"

#include <fstream>

using namespace Lince;

const std::string SetupManager::settingTags[] =
{
	// Strings
	"ServerProxyName", "ServerProxyUsername", "ServerProxyPassword",
	"ClientProxyName", "ClientProxyUsername", "ClientProxyPassword",
	"BindToInterface", "DefaultFileDirectory", "LastSaveDirectory",
	"SENTRY",

	// Ints
	"FirstPort", "LastPort", "DhtSupport", "UpnpSupport", "LocalDiscoverySupport",
	"MetadataExchangeSupport", "PexSupport", "MaxHalfOpenConnections",
	"ServerProxyPort", "ServerUseProxy", "ServerProxyType", "ClientProxyPort",
	"ClientUseProxy", "ClientProxyType", "UploadSpeed", "DisableUploadSpeed",
	"DownloadSpeed", "DisableDownloadSpeed", "UploadSlots", "DisableUploadSlots",
	"ConnectionsNumber", "DisableConnectionsNumber", "ShareRatio_1",  "ShareRatio_2",
	"DisableShareRatio", "AllowConnectionsFromTheSameIP", "PrioritizeFirstPartOfFiles",
	"StorageType", "UseEncryption", "EncryptedIncomingConnections",
	"EncryptedOutgoingConnections", "EncryptionLevel", "EncryptedPreferRC4",
	"ActiveDownloadsValue", "MaxDownloadsValue", "ActiveSharesValue", "UseIpFilter",
	"UseDefaultFileDirecory", "UseScheduler", "SchedulerMonday", "SchedulerTuesday",
	"SchedulerWednesday", "SchedulerThursday", "SchedulerFriday", "SchedulerSaturday",
	"SchedulerSunday", "SchedulerHour0DownSpeed", "SchedulerHour0UpSpeed",
	"SchedulerHour1DownSpeed", "SchedulerHour1UpSpeed", "SchedulerHour2DownSpeed",
	"SchedulerHour2UpSpeed", "SchedulerHour3DownSpeed", "SchedulerHour3UpSpeed",
	"SchedulerHour4DownSpeed", "SchedulerHour4UpSpeed", "SchedulerHour5DownSpeed",
	"SchedulerHour5UpSpeed", "SchedulerHour6DownSpeed", "SchedulerHour6UpSpeed",
	"SchedulerHour7DownSpeed", "SchedulerHour7UpSpeed", "SchedulerHour8DownSpeed",
	"SchedulerHour8UpSpeed", "SchedulerHour9DownSpeed", "SchedulerHour9UpSpeed",
	"SchedulerHour10DownSpeed", "SchedulerHour10UpSpeed", "SchedulerHour11DownSpeed",
	"SchedulerHour11UpSpeed", "SchedulerHour12DownSpeed", "SchedulerHour12UpSpeed",
	"SchedulerHour13DownSpeed", "SchedulerHour13UpSpeed", "SchedulerHour14DownSpeed",
	"SchedulerHour14UpSpeed", "SchedulerHour15DownSpeed", "SchedulerHour15UpSpeed",
	"SchedulerHour16DownSpeed", "SchedulerHour16UpSpeed", "SchedulerHour17DownSpeed",
	"SchedulerHour17UpSpeed", "SchedulerHour18DownSpeed", "SchedulerHour18UpSpeed",
	"SchedulerHour19DownSpeed", "SchedulerHour19UpSpeed", "SchedulerHour20DownSpeed",
	"SchedulerHour20UpSpeed", "SchedulerHour21DownSpeed", "SchedulerHour21UpSpeed",
	"SchedulerHour22DownSpeed", "SchedulerHour22UpSpeed", "SchedulerHour23DownSpeed",
	"SchedulerHour23UpSpeed", "SmartBanSupport",
	"SENTRY",

	// Int64
	"TotalUpload", "TotalDownload", "TotalTime",
	"SENTRY"
};

SetupManagerPtr SetupManager::create ()
{
	return SetupManagerPtr(new SetupManager());
}

SetupManager::SetupManager ()
{
	for(int i=0; i<SETTINGS_LAST; i++)
		isSet[i] = false;

	for(int i=0; i<INT_LAST-INT_FIRST; i++)
	{
		intDefaults[i] = 0;
		intSettings[i] = 0;
	}

	for (int i=0; i < INT64_LAST-INT64_FIRST; i++)
	{
		int64Defaults[i] = 0;
		int64Settings[i] = 0;
	}

	// Strings
	setDefault(SERVER_PROXY_NAME, "");
	setDefault(SERVER_PROXY_USERNAME, "");
	setDefault(SERVER_PROXY_PASSWORD, "");
	setDefault(CLIENT_PROXY_NAME, "");
	setDefault(CLIENT_PROXY_USERNAME, "");
	setDefault(CLIENT_PROXY_PASSWORD, "");
	setDefault(BIND_TO_INTERFACE, "");
	setDefault(DEFAULT_FILE_DIRECTORY, "");
	setDefault(LAST_SAVE_DIRECTORY, "");

	// Ints
	setDefault(FIRST_PORT, 6881);
	setDefault(LAST_PORT, 6889);
	setDefault(DHT_SUPPORT, false);
	setDefault(UPNP_SUPPORT, false);
	setDefault(LOCAL_DISCOVERY_SUPPORT, false);
	setDefault(METADATA_SUPPORT, false);
	setDefault(PEX_SUPPORT, true);
	setDefault(MAX_HALF_OPEN_CONNECTIONS, 0);
	setDefault(SERVER_PROXY_PORT, 8080);
	setDefault(SERVER_USE_PROXY, 0);
	setDefault(SERVER_PROXY_TYPE, 1);
	setDefault(CLIENT_PROXY_PORT, 8080);
	setDefault(CLIENT_USE_PROXY, 0);
	setDefault(CLIENT_PROXY_TYPE, 1);
	setDefault(UPLOAD_SPEED, 1);
	setDefault(DISABLE_UPLOAD_SPEED, true);
	setDefault(DOWNLOAD_SPEED, 1);
	setDefault(DISABLE_DOWNLOAD_SPEED, true);
	setDefault(UPLOAD_SLOTS, 1);
	setDefault(DISABLE_UPLOAD_SLOTS, true);
	setDefault(CONNECTIONS_NUMBER, 1);
	setDefault(DISABLE_CONNECTIONS_NUMBER, true);
	setDefault(SHARE_RATIO_1, 1);
	setDefault(SHARE_RATIO_2, 0);
	setDefault(DISABLE_SHARE_RATIO, true);
	setDefault(CONNECTIONS_FROM_THE_SAME_IP, false);
	setDefault(PRIORITIZE_FIRST_PART_OF_FILES, false);
	setDefault(STORAGE_TYPE, 2);
	setDefault(USE_ENCRYPTION, 0);
	setDefault(ENCRYPTED_INCOMING_CONNECTIONS, 2);
	setDefault(ENCRYPTED_OUTGOING_CONNECTIONS, 2);
	setDefault(ENCRYPTION_LEVEL, 3);
	setDefault(ENCRYPTION_PREFER_RC4, 0);
	setDefault(ACTIVE_DOWNLOADS_VALUE, 0);
	setDefault(MAX_DOWNLOADS_VALUE, 0);
	setDefault(ACTIVE_SHARES_VALUE, 0);
	setDefault(USE_IP_FILTER, true);
	setDefault(USE_DEFAULT_FILE_DIRECTORY, false);
	setDefault(USE_SCHEDULER, false);
	setDefault(SCHEDULER_MONDAY, true);
	setDefault(SCHEDULER_TUESDAY, true);
	setDefault(SCHEDULER_WEDNESDAY, true);
	setDefault(SCHEDULER_THURSDAY, true);
	setDefault(SCHEDULER_FRIDAY, true);
	setDefault(SCHEDULER_SATURDAY, true);
	setDefault(SCHEDULER_SUNDAY, true);
	setDefault(SCHEDULER_HOUR0_DOWN, 0);
	setDefault(SCHEDULER_HOUR0_UP, 0);
	setDefault(SCHEDULER_HOUR1_DOWN, 0);
	setDefault(SCHEDULER_HOUR1_UP, 0);
	setDefault(SCHEDULER_HOUR2_DOWN, 0);
	setDefault(SCHEDULER_HOUR2_UP, 0);
	setDefault(SCHEDULER_HOUR3_DOWN, 0);
	setDefault(SCHEDULER_HOUR3_UP, 0);
	setDefault(SCHEDULER_HOUR4_DOWN, 0);
	setDefault(SCHEDULER_HOUR4_UP, 0);
	setDefault(SCHEDULER_HOUR5_DOWN, 0);
	setDefault(SCHEDULER_HOUR5_UP, 0);
	setDefault(SCHEDULER_HOUR6_DOWN, 0);
	setDefault(SCHEDULER_HOUR6_UP, 0);
	setDefault(SCHEDULER_HOUR7_DOWN, 0);
	setDefault(SCHEDULER_HOUR7_UP, 0);
	setDefault(SCHEDULER_HOUR8_DOWN, 0);
	setDefault(SCHEDULER_HOUR8_UP, 0);
	setDefault(SCHEDULER_HOUR9_DOWN, 0);
	setDefault(SCHEDULER_HOUR9_UP, 0);
	setDefault(SCHEDULER_HOUR10_DOWN, 0);
	setDefault(SCHEDULER_HOUR10_UP, 0);
	setDefault(SCHEDULER_HOUR11_DOWN, 0);
	setDefault(SCHEDULER_HOUR11_UP, 0);
	setDefault(SCHEDULER_HOUR12_DOWN, 0);
	setDefault(SCHEDULER_HOUR12_UP, 0);
	setDefault(SCHEDULER_HOUR13_DOWN, 0);
	setDefault(SCHEDULER_HOUR13_UP, 0);
	setDefault(SCHEDULER_HOUR14_DOWN, 0);
	setDefault(SCHEDULER_HOUR14_UP, 0);
	setDefault(SCHEDULER_HOUR15_DOWN, 0);
	setDefault(SCHEDULER_HOUR16_UP, 0);
	setDefault(SCHEDULER_HOUR17_DOWN, 0);
	setDefault(SCHEDULER_HOUR17_UP, 0);
	setDefault(SCHEDULER_HOUR18_DOWN, 0);
	setDefault(SCHEDULER_HOUR18_UP, 0);
	setDefault(SCHEDULER_HOUR19_DOWN, 0);
	setDefault(SCHEDULER_HOUR19_UP, 0);
	setDefault(SCHEDULER_HOUR20_DOWN, 0);
	setDefault(SCHEDULER_HOUR20_UP, 0);
	setDefault(SCHEDULER_HOUR21_DOWN, 0);
	setDefault(SCHEDULER_HOUR21_UP, 0);
	setDefault(SCHEDULER_HOUR22_DOWN, 0);
	setDefault(SCHEDULER_HOUR22_UP, 0);
	setDefault(SCHEDULER_HOUR23_DOWN, 0);
	setDefault(SCHEDULER_HOUR23_UP, 0);
	setDefault(SMARTBAN_SUPPORT, 0);

	// Int64
	setDefault(TOTAL_UPLOAD, (int64_t) 0);
	setDefault(TOTAL_DOWNLOAD, (int64_t) 0);
	setDefault(TOTAL_TIME, (int64_t) 0);

	config_file = build_filename_as_string(get_base_directory(), "core.xml");
	load();
}

void SetupManager::load()
{
	reader.setFileName(config_file);

	if (reader.findChild("Settings"))
	{
		reader.goToNextElement();

		int i;

		for(i=INT_FIRST; i<INT_LAST; i++)
		{
			const std::string& attr = settingTags[i];

			if(reader.findChildFromBeginning(attr))
            	set(IntSetting(i), to_int(reader.getChildData().c_str()));

		}

		for(i=STR_FIRST; i<STR_LAST; i++)
		{
			const std::string& attr = settingTags[i];

			if(reader.findChildFromBeginning(attr))
            	set(StrSetting(i), reader.getChildData());
		}

		for(i=INT64_FIRST; i<INT64_LAST; i++)
		{
			const std::string& attr = settingTags[i];

			if(reader.findChildFromBeginning(attr))
				set(Int64Setting(i), to_int64(reader.getChildData().c_str()));
		}
	}
}

void SetupManager::save()
{
	writer.setFileName(config_file);
	writer.startDocument("Lince");
	writer.addComment(_("DO NO EDIT THIS FILE BY HAND UNLESS YOU KNOW WHAT ARE YOU DOING!!!"));
	writer.startElement("Settings");

	for(int i=0; i<INT_LAST-INT_FIRST; i++)
	{
		writer.startElement(settingTags[i+INT_FIRST]);
		writer.addAttribute("Type", "int");
		if (!isSet[i+INT_FIRST])
			writer.addElementData(intDefaults[i]);
		else
			writer.addElementData(intSettings[i]);
		writer.endElement();
	}

	for(int i=0; i<STR_LAST-STR_FIRST; i++)
	{
		writer.startElement(settingTags[i]);
		writer.addAttribute("type", "string");
		if (!isSet[i+STR_FIRST])
			writer.addElementData(strDefaults[i]);
		else
			writer.addElementData(strSettings[i]);
		writer.endElement();
	}

	for(int i=0; i<INT64_LAST-INT64_FIRST; i++)
	{
		writer.startElement(settingTags[i+INT64_FIRST]);
		writer.addAttribute("type", "int64");
		if (!isSet[i+INT64_FIRST])
			writer.addElementData(int64Defaults[i]);
		else
			writer.addElementData(int64Settings[i]);
		writer.endElement();
	}

	writer.endDocument();
}

SetupManager::~SetupManager ()
{
	save();
}
