/***************************************************************************
 *            TorrentsManager.cc
 *
 *  Copyright  2008  Fernando Tarín Morales
 *  icemanf@gmail.com
 ****************************************************************************/

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */
 
#include "Lince/TorrentsManager.h"

#include <utility>

#include "Lince/Core.h"
#include "Lince/SetupManager.h"
#include "Lince/AlertsManager.h"

using namespace Lince;

TorrentsManagerPtr TorrentsManager::create()
{
	return TorrentsManagerPtr(new TorrentsManager());
}

TorrentsManager::TorrentsManager()
{
	Core::get_alerts_manager()->
		finished_torrent_signal(boost::bind(&TorrentsManager::on_finished_torrent, this, _1));
	Core::get_alerts_manager()->
		torrent_stopped_signal2(boost::bind(&TorrentsManager::on_stopped_torrent, this, _1));
	Core::get_alerts_manager()->
		torrent_started_signal2(boost::bind(&TorrentsManager::on_started_torrent, this, _1));
}

// Signals
void TorrentsManager::on_finished_torrent(int position)
{
	Torrent temp;
	torrentList::iterator i;
	SetupManagerPtr s = Core::get_setup_manager();
		
	i = get_downloading_torrent_iterator_at(position);
	temp = *i;
	
	remove_downloading_torrent_at(position);
	
	temp.set_complete();
	if (temp.is_queued())
		temp.unqueue();
	
	append_shared_torrent(temp);
	m_signal_torrent_switched(position);
}

void TorrentsManager::on_stopped_torrent(const torrentPosition &tp)
{
	Torrent *t = get_torrent_at(tp);
	SetupManagerPtr s = Core::get_setup_manager();
	
	if (!tp.second && s->getBool(SetupManager::ACTIVE_DOWNLOADS_VALUE))
	{
		if (t->is_queued())
		{
			queuedDownloadingTorrents.remove(t->position());
			t->unqueue();
		}
		else
		{
			activeDownloadingTorrents.remove(t->position());
			
			if (activeDownloadingTorrents.size() <= s->get(SetupManager::ACTIVE_DOWNLOADS_VALUE)
				&& queuedDownloadingTorrents.size())
			{
					queuedDownloadingTorrents.sort();
					downloadingTorrents[queuedDownloadingTorrents.front() - 1].unqueue();
					activeDownloadingTorrents.push_back(queuedDownloadingTorrents.front());
					queuedDownloadingTorrents.pop_front();
			}
		}
		
		stoppedDownloadingTorrents.push_back(t->position());
	}
	else if (tp.second && s->getBool(SetupManager::ACTIVE_SHARES_VALUE))
	{
		if (t->is_queued())
		{
			queuedSharedTorrents.remove(t->position());
			t->unqueue();
		}
		else
		{
			activeSharedTorrents.remove(t->position());
			
			if (activeSharedTorrents.size() <= s->get(SetupManager::ACTIVE_SHARES_VALUE)
				&& queuedSharedTorrents.size())
			{
					queuedSharedTorrents.sort();
					sharedTorrents[queuedSharedTorrents.front() - 1].unqueue();
					activeSharedTorrents.push_back(queuedSharedTorrents.front());
					queuedSharedTorrents.pop_front();
			}
		}
		
		stoppedSharedTorrents.push_back(t->position());
	}
}

void TorrentsManager::on_started_torrent(const torrentPosition &tp)
{
	Torrent *t = get_torrent_at(tp);
	SetupManagerPtr s = Core::get_setup_manager();
	
	if (!tp.second && s->getBool(SetupManager::ACTIVE_DOWNLOADS_VALUE))
	{
		stoppedDownloadingTorrents.remove(t->position());
		
		if (activeDownloadingTorrents.size() <= s->get(SetupManager::ACTIVE_DOWNLOADS_VALUE))
		{
			activeDownloadingTorrents.push_back(t->position());
		}
		else
		{
			queuedDownloadingTorrents.push_back(t->position());
			t->queue();
		}
		
		if (activeDownloadingTorrents.size() > s->get(SetupManager::ACTIVE_DOWNLOADS_VALUE))
		{
			activeDownloadingTorrents.sort();
			downloadingTorrents[activeDownloadingTorrents.back() - 1].queue();
			queuedDownloadingTorrents.push_back(activeDownloadingTorrents.back());
			activeDownloadingTorrents.pop_back();
		}
	}
	else if(tp.second && s->getBool(SetupManager::ACTIVE_SHARES_VALUE))
	{
		stoppedSharedTorrents.remove(t->position());
		
		if (activeSharedTorrents.size() <= s->get(SetupManager::ACTIVE_SHARES_VALUE))
		{
			activeSharedTorrents.push_back(t->position());
		}
		else
		{
			queuedSharedTorrents.push_back(t->position());
			t->queue();
		}
		
		if (activeSharedTorrents.size() > s->get(SetupManager::ACTIVE_SHARES_VALUE))
		{
			activeSharedTorrents.sort();
			sharedTorrents[activeSharedTorrents.back() - 1].queue();
			queuedSharedTorrents.push_back(activeSharedTorrents.back());
			activeSharedTorrents.pop_back();
		}
	}
}

torrentPosition TorrentsManager::get_torrent_position(const libtorrent::sha1_hash &hash)
{
	torrentPositions::iterator iter;
	iter = tPositions.find(hash);
	
	if (iter != tPositions.end())
		return iter->second;
	
	return std::make_pair(-1, false);
}

Torrent * TorrentsManager::get_torrent_at(const torrentPosition &tp)
{
	// Downloading torrent
	if (!tp.second)
		return &downloadingTorrents.at(tp.first-1);
	// Shared torrent
	else
		return &sharedTorrents.at(tp.first-1);
}

void TorrentsManager::start_all_torrents()
{
	start_all_downloading_torrents();
	start_all_shared_torrents();
}

void TorrentsManager::stop_all_torrents()
{
	stop_all_downloading_torrents();
	stop_all_shared_torrents();
}
	

// Downloading torrents
void TorrentsManager::append_downloading_torrent(Torrent& t)
{
	SetupManagerPtr s = Core::get_setup_manager();
	
	// Set position
	t.set_position(downloadingTorrents.size() + 1);
	
	// Manage queues
	if (s->getBool(SetupManager::ACTIVE_DOWNLOADS_VALUE))
	{
		if (t.is_stopped())
		{
			stoppedDownloadingTorrents.push_back(t.position());
		}
		else
		{
			if (activeDownloadingTorrents.size() < s->get(SetupManager::ACTIVE_DOWNLOADS_VALUE))
			{
				activeDownloadingTorrents.push_back(t.position());
			}
			else
			{
				queuedDownloadingTorrents.push_back(t.position());
				t.queue();
			}
		}
	}	
	
	// Add torrent position.
	tPositions[t.info_hash()] = std::make_pair(t.position(), false);
	
	downloadingTorrents.push_back(t);
}

void TorrentsManager::remove_downloading_torrent_at(int position)
{
	SetupManagerPtr s = Core::get_setup_manager();
	torrentList::iterator i;
	
	// Update positions
	if (downloadingTorrents.size() > 1)
	{
		for (i=get_downloading_torrent_iterator_at(position+1); i != get_downloading_torrent_iterator_at_end(); ++i)
		{
			i->set_position(i->position()-1);
			tPositions[i->info_hash()] = std::make_pair(i->position(), false);
		}
	}
	
	tPositions.erase(get_downloading_torrent_iterator_at(position)->info_hash());
	downloadingTorrents.erase(get_downloading_torrent_iterator_at(position));
	
	// Update queues
	if (s->getBool(SetupManager::ACTIVE_DOWNLOADS_VALUE))
		rebuild_downloading_torrents_queue();
}

Torrent* TorrentsManager::get_downloading_torrent_at(int position)
{
	return &downloadingTorrents.at(position-1);
}

torrentList::iterator TorrentsManager::get_downloading_torrent_iterator_at(int position)
{
	torrentList::iterator i = downloadingTorrents.begin();
	i += (position-1);
	return i;
}

torrentList::iterator TorrentsManager::get_downloading_torrent_iterator_at_begin()
{
	return downloadingTorrents.begin();
}

torrentList::iterator TorrentsManager::get_downloading_torrent_iterator_at_end()
{
	return downloadingTorrents.end();
}

void TorrentsManager::move_downloading_torrent(int from, int to)
{
	Torrent temp;
	torrentList::iterator last;
	SetupManagerPtr s = Core::get_setup_manager();
	
	if (from == to || to > downloading_torrents_count() || from == 0
		|| to == 0 || from > downloading_torrents_count()) 
			return;
	
	activeDownloadingTorrents.sort();
	
	temp = *(get_downloading_torrent_iterator_at(from));
	temp.set_position(to);
	tPositions[temp.info_hash()] = std::make_pair(temp.position(), false);
	
	if (to > from)
		last = get_downloading_torrent_iterator_at(to+1);
	else
		last = get_downloading_torrent_iterator_at(to);
	
	downloadingTorrents.insert(last, temp);
		
	if (to > from)
		downloadingTorrents.erase(get_downloading_torrent_iterator_at(from));
	else
		downloadingTorrents.erase(get_downloading_torrent_iterator_at(from + 1));
	
	if (to > from)
	{
		for (torrentList::iterator i = get_downloading_torrent_iterator_at(from);
			 i != get_downloading_torrent_iterator_at(to); ++i)
		{
			i->set_position(i->position()-1);
			tPositions[i->info_hash()] = std::make_pair(i->position(), false);
		}
	}
	else
	{
		for (torrentList::iterator i = get_downloading_torrent_iterator_at(to + 1);
			 i != get_downloading_torrent_iterator_at(from + 1); ++i)
		{
			i->set_position(i->position()+1);
			tPositions[i->info_hash()] = std::make_pair(i->position(), false);
		}
	}
	
	if (s->getBool(SetupManager::ACTIVE_DOWNLOADS_VALUE))
	{
		rebuild_downloading_torrents_queue();
	}
}

void TorrentsManager::start_downloading_torrent_at(int position)
{
	get_downloading_torrent_iterator_at(position)->start();
}


void TorrentsManager::start_all_downloading_torrents()
{
	torrentList::iterator iter;	
	for (iter=downloadingTorrents.begin(); iter != downloadingTorrents.end(); ++iter)
		iter->start();
}		


void TorrentsManager::stop_downloading_torrent_at(int position)
{
	get_downloading_torrent_iterator_at(position)->stop();
}


void TorrentsManager::stop_all_downloading_torrents()
{
	torrentList::iterator iter;	
	for (iter=downloadingTorrents.begin(); iter != downloadingTorrents.end(); ++iter)
		iter->stop();
}

void TorrentsManager::rebuild_downloading_torrents_queue()
{	
	SetupManagerPtr s = Core::get_setup_manager();
	torrentList::iterator iter;
	
	stoppedDownloadingTorrents.clear(); 
	activeDownloadingTorrents.clear();
	queuedDownloadingTorrents.clear();
	
	for (iter = get_downloading_torrent_iterator_at_begin();
		 iter != get_downloading_torrent_iterator_at_end();
		 ++iter)
	{
		if (!s->getBool(SetupManager::ACTIVE_DOWNLOADS_VALUE))
		{
			if (iter->is_queued())
					iter->unqueue();
		}
		else{
			if (iter->is_stopped())
				stoppedDownloadingTorrents.push_back(iter->position());
			
			else{
				if (activeDownloadingTorrents.size() < s->get(SetupManager::ACTIVE_DOWNLOADS_VALUE))
				{
					activeDownloadingTorrents.push_back(iter->position());
					if (iter->is_queued())
						iter->unqueue();
				}
				else
				{
					queuedDownloadingTorrents.push_back(iter->position());
					if (!iter->is_queued())
						iter->queue();
				}
			}
		}
	}
}


// Shared torrents
void TorrentsManager::append_shared_torrent(Torrent& t)
{
	SetupManagerPtr s = Core::get_setup_manager();
	
	// Set position
	t.set_position(sharedTorrents.size() + 1);
	
	// Manage queues
	if (s->getBool(SetupManager::ACTIVE_SHARES_VALUE))
	{
		if (t.is_stopped())
		{
			stoppedSharedTorrents.push_back(t.position());
		}
		else
		{
			if (activeSharedTorrents.size() < s->get(SetupManager::ACTIVE_SHARES_VALUE))
				activeSharedTorrents.push_back(t.position());
			else
			{
				queuedSharedTorrents.push_back(t.position());
				t.queue();
			}
		}
	}
	
	// Add torrent position.
	tPositions[t.info_hash()] = std::make_pair(t.position(), true);
	
	sharedTorrents.push_back(t);
}

void TorrentsManager::remove_shared_torrent_at(int position)
{
	SetupManagerPtr s = Core::get_setup_manager();
	torrentList::iterator i;
	
	// Update positions
	if (sharedTorrents.size() > 1)
	{
		for (i=get_shared_torrent_iterator_at(position+1); i != get_shared_torrent_iterator_at_end() ;++i)
		{
			i->set_position(i->position()-1);
			tPositions[i->info_hash()] = std::make_pair(i->position(), true);
		}
	}
		
	tPositions.erase(get_shared_torrent_iterator_at(position)->info_hash());
	sharedTorrents.erase(get_shared_torrent_iterator_at(position));
	
	if (s->getBool(SetupManager::ACTIVE_SHARES_VALUE))
		rebuild_shared_torrents_queue();
}

Torrent* TorrentsManager::get_shared_torrent_at(int position)
{
	return &sharedTorrents.at(position-1);
}

torrentList::iterator TorrentsManager::get_shared_torrent_iterator_at_begin()
{
	return sharedTorrents.begin();
}

torrentList::iterator TorrentsManager::get_shared_torrent_iterator_at_end()
{
	return sharedTorrents.end();
}
		
torrentList::iterator TorrentsManager::get_shared_torrent_iterator_at(int position)
{
	torrentList::iterator i = sharedTorrents.begin();
	i += (position-1);
	return i;
}

void TorrentsManager::move_shared_torrent(int from, int to)
{
	Torrent temp;
	torrentList::iterator last;
	SetupManagerPtr s = Core::get_setup_manager();
	
	if (from == to || to > shared_torrents_count() || from == 0
		|| to == 0 || from > shared_torrents_count()) 
			return;
	
	activeSharedTorrents.sort();
	
	temp = *(get_shared_torrent_iterator_at(from));
	temp.set_position(to);
	tPositions[temp.info_hash()] = std::make_pair(temp.position(), true);
	
	if (to > from)
		last = get_shared_torrent_iterator_at(to+1);
	else
		last = get_shared_torrent_iterator_at(to);

	
	sharedTorrents.insert(last, temp);
		
	if (to > from)
		sharedTorrents.erase(get_shared_torrent_iterator_at(from));
	else
		sharedTorrents.erase(get_shared_torrent_iterator_at(from + 1));
	
	if (to > from)
	{
		for (torrentList::iterator i = get_shared_torrent_iterator_at(from);
			 i != get_shared_torrent_iterator_at(to); ++i)
		{
			i->set_position(i->position()-1);
			tPositions[i->info_hash()] = std::make_pair(i->position(), true);
		}
	}
	else
	{
		for (torrentList::iterator i = get_shared_torrent_iterator_at(to + 1);
			 i != get_shared_torrent_iterator_at(from + 1); ++i)
		{
			i->set_position(i->position()+1);
			tPositions[i->info_hash()] = std::make_pair(i->position(), true);
		}
	}
	
	if (s->getBool(SetupManager::ACTIVE_SHARES_VALUE))
	{
		rebuild_shared_torrents_queue();
	}
}


void TorrentsManager::start_shared_torrent_at(int position)
{
	get_shared_torrent_iterator_at(position)->start();
}


void TorrentsManager::start_all_shared_torrents()
{
	torrentList::iterator iter;	
	for (iter=sharedTorrents.begin(); iter != sharedTorrents.end(); ++iter)
		iter->start();
}


void TorrentsManager::stop_shared_torrent_at(int position)
{
	get_shared_torrent_iterator_at(position)->stop();
}


void TorrentsManager::stop_all_shared_torrents()
{
	torrentList::iterator iter;	
	for (iter=sharedTorrents.begin(); iter != sharedTorrents.end(); ++iter)
		iter->stop();
}

void TorrentsManager::rebuild_shared_torrents_queue()
{	
	SetupManagerPtr s = Core::get_setup_manager();
	torrentList::iterator iter;
	
	stoppedSharedTorrents.clear(); 
	activeSharedTorrents.clear();
	queuedSharedTorrents.clear();
	
	for (iter = get_shared_torrent_iterator_at_begin();
		 iter != get_shared_torrent_iterator_at_end();
		 ++iter)
	{
		if (!s->getBool(SetupManager::ACTIVE_SHARES_VALUE))
		{
			if (iter->is_queued())
					iter->unqueue();
		}
		else{
			if (iter->is_stopped())
				stoppedSharedTorrents.push_back(iter->position());
			
			else{
				if (activeSharedTorrents.size() < s->get(SetupManager::ACTIVE_SHARES_VALUE))
				{
					activeSharedTorrents.push_back(iter->position());
					if (iter->is_queued())
						iter->unqueue();
				}
				else
				{
					queuedSharedTorrents.push_back(iter->position());
					if (!iter->is_queued())
						iter->queue();
				}
			}
		}
	}	
}

TorrentsManager::~TorrentsManager()
{
}

